from OpenGL.GL import *

class CompileError (Exception):
	def __init__(self, shader):
		self.shader = shader

class Shader (object):
	def __init__(self, type, source):
		self.shader = glCreateShader(type)

		if source:
			self.compile(source)

	def compile(self, source):
		glShaderSource(self.shader, source)
		glCompileShader(self.shader)

		if not self.is_compiled():
			raise CompileError(self)

	def get_log(self):	
		return glGetShaderInfoLog(self.shader)

	def is_compiled(self):
		return glGetShaderiv(self.shader, GL_COMPILE_STATUS)

class VertexShader (Shader):
	def __init__(self, source):
		Shader.__init__(self, GL_VERTEX_SHADER, source)
	
class FragmentShader (Shader):
	def __init__(self, source):
		Shader.__init__(self, GL_FRAGMENT_SHADER, source)

class LinkError (Exception):
	def __init__(self, program):
		self.program = program

class Program (object):
	def __init__(self, *shaders):
		self.program = glCreateProgram()
		self.attach(*shaders)
	
	def attach(self, *shaders):
		for shader in list(shaders):
			if hasattr(shader, 'shader'):
				shader = shader.shader
			
			print shader
			glAttachShader(self.program, shader)
	
	def link(self):
		glLinkProgram(self.program)

		if not self.is_linked():
			raise LinkError(self)
	
	def is_linked(self):
		return glGetProgramiv(self.program, GL_LINK_STATUS)

	def get_log(self):
		return glGetProgramInfoLog(self.program)

	def use(self):
		glUseProgram(self.program)

if __name__ == '__main__':
	vertexshader = """
	void main(void)
	{
		vec4 a = gl_Vertex;
		a.x = a.x * 0.5;
		a.y = a.y * 0.5;

		gl_Position = gl_ModelViewProjectionMatrix * a;
	}
	"""

	fragmentshader = """
	void main(void)
	{
		gl_FragColor = vec4(0.0, 1.0, 0.0, 1.0);
	}
	"""

	import sys, pygame
	from pygame.locals import *

	disp = pygame.display.set_mode((1024, 768), OPENGL | DOUBLEBUF)
	
	def try_shader(klass, source):
		try:
			return klass(source)
		except CompileError, ex:
			print 'Failed to compile shader.'
			print ex.shader.get_log()
	
	vs = try_shader(VertexShader, vertexshader)
	fs = try_shader(FragmentShader, fragmentshader)

	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)

	prog = Program(vs, fs)
	try:
		prog.link()
	except LinkError, ex:
		print 'Failed to link program.'
		print ex.program.get_log()

	prog.use()
	
	while 1:
		if pygame.event.get(QUIT):
			break

		glClear(GL_COLOR_BUFFER_BIT)
		glBegin(GL_TRIANGLES)
		glVertex3f(1.0, 1.0, 0)
		glVertex3f(-1.0, 1.0, 0)
		glVertex3f(0, 0, 0)
		glEnd()
		
		pygame.display.flip()

